use std::collections::HashMap;

use std::time::Duration;

pub struct Timings {
    interval: Duration,
    last: Duration,
    timings: HashMap<String, Vec<Duration>>,
}

impl Default for Timings {
    fn default() -> Self {
        Self {
            interval: Duration::from_secs(1),
            last: Default::default(),
            timings: Default::default(),
        }
    }
}

impl Timings {
    pub fn add_timing(mut self, name: impl ToString) -> Self {
        self.timings.insert(name.to_string(), Vec::new());

        self
    }

    pub fn set_interval(mut self, interval: Duration) -> Self {
        self.interval = interval;

        self
    }

    pub fn add_timing_afterwards(&mut self, name: impl ToString) {
        self.timings.insert(name.to_string(), Vec::new());
    }

    pub fn check_timing(
        &mut self,
        now: Duration,
        filter: impl Into<Option<Duration>>,
    ) -> Option<Vec<(String, Duration)>> {
        if now >= self.last + self.interval {
            self.last = now;

            let mut result = Vec::new();

            for (name, values) in self.timings.iter_mut() {
                let average = if values.is_empty() {
                    Duration::from_secs(1)
                } else {
                    let sum: Duration = values.iter().sum();
                    let average = sum / values.len() as u32;

                    *values = Vec::new();

                    average
                };

                result.push((name.clone(), average));
            }

            if let Some(filter) = filter.into() {
                result.retain(|(_, duration)| *duration > filter);
            }

            result.sort_by(|(_, lhs), (_, rhs)| lhs.cmp(rhs));

            return Some(result);
        }

        None
    }

    pub fn add(&mut self, name: &str, timing: Duration) {
        if let Some(values) = self.timings.get_mut(name) {
            values.push(timing);
        }
    }

    pub fn clear(&mut self) {
        self.timings.clear();
    }
}
